#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{

    /// <summary>
	/// ProfitTargetTrailingStop_1.01b
    /// Adds a Stop Limit, Profit Target and Trailing Stop that advances on close of bar or every tick according to CalculateOnCloseOfBar true or false
    /// </summary>
    [Description("ProfitTargetTrailingStop Version 1.01b. StopLoss, Trailing Stop and ProfitTarget With Controls. By Chris Long. alcamie@gmail.com")]
    public class TrailingStopSwitchExit : Strategy
    {
        #region Variables
		private int		breakEvenTicks		= 4;		// Default setting for ticks needed to acheive before stop moves to breakeven		
		private int		plusBreakEven		= 2; 		// Default setting for amount of ticks past breakeven to actually breakeven
		private int		profitTargetTicks	= 40;		// Default setting for how many Ticks away from AvgPrice is profit target
        private int		stopLossTicks		= 15;		// Default setting for stoploss. Ticks away from AvgPrice		
		private int		trailProfitTrigger	= 10;		// 8 Default Setting for trail trigger ie the number of ticks movede after break even befor activating TrailStep
		private int		trailStepTicks		= 8;		// 2 Default setting for number of ticks advanced in the trails - take into consideration the barsize as is calculated/advanced next bar
		private int 	BarTraded 			= 0; 		// Default setting for Bar number that trade occurs		
		
		private double	initialBreakEven	= 0; 		// Default setting for where you set the breakeven
		private double 	previousPrice		= 0;		// previous price used to calculate trailing stop
		private double 	newPrice			= 0;		// Default setting for new price used to calculate trailing stop

		#endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
		#region Initialization Functions
        protected override void Initialize()
        {
			/* There are several ways you can use SetStopLoss and SetProfitTarget. You can have them set to a currency value
			or some sort of calculation mode. Calculation modes available are by percent, price, and ticks. SetStopLoss and
			SetProfitTarget will submit real working orders unless you decide to simulate the orders. */
			
			SetStopLoss(CalculationMode.Ticks, stopLossTicks);
			SetProfitTarget(CalculationMode.Ticks, profitTargetTicks);

            CalculateOnBarClose = true;
			ExitOnClose = false;
			
			//Debugging
			TraceOrders        = true; 
        }
		#endregion
			
		#region Entry Functions
			
			
			private void FillLongEntry1()
			{
				EnterLong();
				BarTraded = CurrentBar;
			}
			
			private void FillShortEntry1()
			{
				EnterShort();
				BarTraded = CurrentBar;
			}
			
	
		#endregion
			
			
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
////			// Make sure this strategy does not execute against historical data
////				if (Historical)
////					return;
			
			#region Breakeven and Trailing Stops
			// Resets the stop loss to the original value when all positions are closed
			   switch (Position.MarketPosition)
            {
                case MarketPosition.Flat:
                    SetStopLoss(CalculationMode.Ticks, stopLossTicks);
					previousPrice = 0;
                    break;
                case MarketPosition.Long:
                    // Once the price is greater than entry price+ breakEvenTicks ticks, set stop loss to breakeven
                    if (Close[0] > Position.AvgPrice + breakEvenTicks * TickSize
						&& previousPrice == 0)
                    {
						initialBreakEven = Position.AvgPrice + plusBreakEven * TickSize;
                        SetStopLoss(CalculationMode.Price, initialBreakEven);
						previousPrice = Position.AvgPrice;
						PrintWithTimeStamp("previousPrice = "+previousPrice);
                    }
					// Once at breakeven wait till trailProfitTrigger is reached before advancing stoploss by trailStepTicks size step
					else if (previousPrice	!= 0 ////StopLoss is at breakeven
 							&& GetCurrentAsk() > previousPrice + trailProfitTrigger * TickSize
						)
					{
						newPrice = previousPrice + trailStepTicks * TickSize;
						SetStopLoss(CalculationMode.Price, newPrice);
						previousPrice = newPrice;
						PrintWithTimeStamp("previousPrice = "+previousPrice);
					}
                    break;
                case MarketPosition.Short:
                    // Once the price is Less than entry price - breakEvenTicks ticks, set stop loss to breakeven
                    if (Close[0] < Position.AvgPrice - breakEvenTicks * TickSize
						&& previousPrice == 0)
                    {
						initialBreakEven = Position.AvgPrice - plusBreakEven * TickSize;
                        SetStopLoss(CalculationMode.Price, initialBreakEven);
						previousPrice = Position.AvgPrice;
						PrintWithTimeStamp("previousPrice = "+previousPrice);
                    }
					// Once at breakeven wait till trailProfitTrigger is reached before advancing stoploss by trailStepTicks size step
					else if (previousPrice	!= 0 ////StopLoss is at breakeven
 							&& GetCurrentAsk() < previousPrice - trailProfitTrigger * TickSize
						)
					{
						newPrice = previousPrice - trailStepTicks * TickSize;
						SetStopLoss(CalculationMode.Price, newPrice);
						previousPrice = newPrice;
						PrintWithTimeStamp("previousPrice = "+previousPrice);
					}
                    break;
                default:
					//autoStopA900Traded = false;
                    break;
			}
			#endregion
			
			#region Enter Trades
            // LongEntry
            if (
				Position.MarketPosition != MarketPosition.Short
                && GetCurrentAsk() >= Close[1]
				&& CurrentBar > BarTraded
				)
            {	
				FillLongEntry1();
            }

		    // ShortEntry
            if (
				Position.MarketPosition != MarketPosition.Long
                && GetCurrentAsk() <= Close[1]
				&& CurrentBar > BarTraded
				)
            {	
				FillShortEntry1();
            }

			#endregion

		}

        #region Properties
		
		[Description("Number of ticks past breakeven to actually breakeven")]
		[Category("Parameters")]
		public int PlusBreakEven
		{
			get { return plusBreakEven; }
			set { plusBreakEven = Math.Max(0, value); }
		}		
		
		[Description("Numbers of ticks away from entry price for the Stop Loss order")]
		[Category("Parameters")]
		public int StopLossTicks
		{
			get { return stopLossTicks; }
			set { stopLossTicks = Math.Max(0, value); }
		}
		
		[Description("Number of ticks away from entry price for the Profit Target order")]
		[Category("Parameters")]
		public int ProfitTargetTicks
		{
			get { return profitTargetTicks; }
			set { profitTargetTicks = Math.Max(0, value); }
		}
		
		[Description("Number of ticks away from break even")]
		[Category("Parameters")]
		public int BreakEvenTicks
		{
			get {return breakEvenTicks;}
			set {breakEvenTicks = Math.Max(0, value);}
		}
		
		[Description("Number of ticks to step for trail")]
		[Category("Parameters")]
		public int TrailStepTicks
		{
			get {return trailStepTicks;}
			set {trailStepTicks = Math.Max(0, value);}
		}
		
		[Description("Number of ticks to trigger trail step")]
		[Category("Parameters")]
		public int TrailProfitTrigger
		{
			get {return trailProfitTrigger;}
			set {trailProfitTrigger = Math.Max(0, value);}
		}

        #endregion
    }
}

